Attribute VB_Name = "modErrMsgs"
' ************************************************************
' Copyright  1995-2001 Slightly Tilted Software
' All rights reserved
' You're absolutely free to use these resources within your
'     own applications, but you may not redistribute them
'     (as source) in any manner whatsoever, whether for profit
'     or not.
' The only legitimate source for the original source code is
'     at the VBPJ site and my own web site at:
'     http://www.SlightlyTiltedSoftware.com
' ************************************************************

' ************************************************************
' NOTE: The resource file used by this module (NetMsg.res) and
'       the function, GetMsgTableResEntry, were supplied by
'       Brad Martinez. The original can be found at:
'           http://www.mvps.org/btmtz/_misc/netmsg.zip
'       and it is used with permission.
'
'       Copyright  1997-2001 Brad Martinez, http://www.mvps.org
'
'       I modified the coding conventions of the function
'       to match the rest of the code, and any errors
'       encounter here should be directed to me instead of Brad.
'
'       If you use the original code instead, write Brad <g>.
' ************************************************************

' ---------------------------------------------
' Module    : modNetCommon.BAS
' By        : L.J. Johnson       Date: 09-23-2001
' Comments  : Uses API to get text strings for
'           :     API error numbers.
' ---------------------------------------------
Option Explicit

Public Const LANG_USER_DEFAULT               As Long = &H400&

' ---------------------------------------------
' Used to get error messages directly from the
'    system instead of hard-coding them
' ---------------------------------------------
Private Const FORMAT_MESSAGE_FROM_SYSTEM     As Long = &H1000&
Private Const FORMAT_MESSAGE_IGNORE_INSERTS  As Long = &H200&
Private Const FORMAT_MESSAGE_FROM_HMODULE    As Long = &H800&
Private Const LOAD_LIBRARY_AS_DATAFILE       As Long = 2&
Private Const MAX_PATH                       As Long = 260&

' ---------------------------------------------
' Upper and lower bounds of network errors
' ---------------------------------------------
Private Const NERR_BASE                      As Long = 2100&
Private Const MAX_NERR                       As Long = NERR_BASE + 899&

' ---------------------------------------------
' Upper and lower bounds of Internet errors
' ---------------------------------------------
Private Const INTERNET_ERROR_BASE            As Long = 12000&
Private Const INTERNET_ERROR_LAST            As Long = INTERNET_ERROR_BASE + 171&

Private Declare Function FormatMessage _
   Lib "kernel32" Alias "FormatMessageA" _
   (ByVal dwFlags As Long, _
    lpSource As Any, _
    ByVal dwMessageId As Long, _
    ByVal dwLanguageId As Long, _
    ByVal lpBuffer As String, _
    ByVal nSize As Long, _
    Arguments As Long) As Long
Private Declare Function LoadLibraryEx _
   Lib "kernel32" Alias "LoadLibraryExA" _
   (ByVal lpLibFileName As String, _
    ByVal hFile As Long, _
    ByVal dwFlags As Long) As Long
Private Declare Function FreeLibrary _
   Lib "kernel32" _
   (ByVal hLibModule As Long) As Long

Public Type MESSAGE_RESOURCE_BLOCK
  LowId                                As Long
  HighId                               As Long
  OffsetToEntries                      As Long
End Type

Public Type MESSAGE_RESOURCE_ENTRY
  Length                               As Integer
  Flags                                As Integer
  Text                                 As Byte        ' get string from VarPtr(mrb.OffsetToEntries) + CB_MRE_TEXTOFFSET
  pad(0 To 2)                          As Byte        ' DWORD align the struct
End Type

Public Const CB_MRB                    As Long = 12   ' user-defined
Public Const CB_MRE                    As Long = 8    ' user-defined
Public Const CB_MRE_TEXTOFFSET         As Long = 4    ' user-defined

' *******************************************************
' Routine Name : (PUBLIC in MODULE) Function ReturnApiErrString
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Written : 01/16/1999 -- 12:56:46
' Inputs       : ErrorCode:Long - Number returned from API error
' Outputs      : N/A
' Description  : Function returns the error string
'              : The original code appeared in Keith Pleas
'              :     article in VBPJ, April 1996 (OLE Expert
'              :     column).  Thanks, Keith.
'              : If the error is from 2100 to 2999, *and*
'              :     if it's running under Win9x (optional
'              :     parameter xi_blnIsWin9x = TRUE), then
'              :     use the NetMsg.res file to return
'              :     the error string instead of loading
'              :     netmsg.dll (which isn't loaded by
'              :     on Win9x systems default)
'              : Brad Martinez created the resource file,
'              :     and the GetMsgTableResEntry function
'              :     (see text at the beginning of the
'              :      module for a fuller description)
' *******************************************************
Public Function ReturnApiErrString(ErrorCode As Long, _
                                   Optional ByVal xi_blnIsWin9x As Boolean = False) As String
On Error Resume Next                   ' Don't accept an error here
   Dim p_strBuffer                     As String
   Dim p_lngHwndModule                 As Long
   Dim p_lngFlags                      As Long
   Dim p_lngPos                        As Long
   Dim p_abytResData()                 As Byte
   
   ' ------------------------------------------
   ' Separate handling for network errors
   ' netmsg.dll
   ' ------------------------------------------
   If ErrorCode >= NERR_BASE And _
      ErrorCode <= MAX_NERR Then
      
      If xi_blnIsWin9x = True Then
         p_abytResData = LoadResData(1, 11)
         p_strBuffer = GetMsgTableResEntry(p_abytResData, ErrorCode)
         
      Else
         p_lngHwndModule = LoadLibraryEx(lpLibFileName:="netmsg.dll", _
                                         hFile:=0&, _
                                         dwFlags:=LOAD_LIBRARY_AS_DATAFILE)
      
         If p_lngHwndModule <> 0 Then
         
            p_lngFlags = FORMAT_MESSAGE_FROM_SYSTEM Or _
                         FORMAT_MESSAGE_IGNORE_INSERTS Or _
                         FORMAT_MESSAGE_FROM_HMODULE
                         
            ' ------------------------------------
            ' Allocate the string, then get the
            '     system to tell us the error
            '     message associated with this error number
            ' ------------------------------------
            p_strBuffer = String(256, 0)
            FormatMessage dwFlags:=p_lngFlags, _
                          lpSource:=ByVal p_lngHwndModule, _
                          dwMessageId:=ErrorCode, _
                          dwLanguageId:=0&, _
                          lpBuffer:=p_strBuffer, _
                          nSize:=Len(p_strBuffer), _
                          Arguments:=ByVal 0&
         
            FreeLibrary hLibModule:=p_lngHwndModule
         End If
      
      End If
   
   ' ------------------------------------------
   ' Separate handling for Wininet error
   ' Wininet.dll
   ' ------------------------------------------
   ElseIf ErrorCode >= INTERNET_ERROR_BASE And _
          ErrorCode <= INTERNET_ERROR_LAST Then
      
      ' ---------------------------------------
      ' Load the library
      ' ---------------------------------------
      p_lngHwndModule = LoadLibraryEx(lpLibFileName:="Wininet.dll", _
                        hFile:=0&, _
                        dwFlags:=LOAD_LIBRARY_AS_DATAFILE)
      
      If p_lngHwndModule <> 0 Then
      
         p_lngFlags = FORMAT_MESSAGE_FROM_SYSTEM Or _
                      FORMAT_MESSAGE_IGNORE_INSERTS Or _
                      FORMAT_MESSAGE_FROM_HMODULE
                      
         ' ------------------------------------
         ' Allocate the string, then get the
         '     system to tell us the error
         '     message associated with this error number
         ' ------------------------------------
         p_strBuffer = String(256, 0)
         FormatMessage dwFlags:=p_lngFlags, _
                       lpSource:=ByVal p_lngHwndModule, _
                       dwMessageId:=ErrorCode, _
                       dwLanguageId:=0&, _
                       lpBuffer:=p_strBuffer, _
                       nSize:=Len(p_strBuffer), _
                       Arguments:=ByVal 0&
      
         FreeLibrary hLibModule:=p_lngHwndModule
      End If
   
   ' ------------------------------------------
   ' Wasn't Wininet or NetMsg, so do the standard
   '     API error look-up
   ' ------------------------------------------
   Else
      ' ---------------------------------------
      ' Allocate the string, then get the system
      '     to tell us the error message associated
      '     with this error number
      ' ---------------------------------------
      p_strBuffer = String(256, 0)
      p_lngFlags = FORMAT_MESSAGE_FROM_SYSTEM Or _
                   FORMAT_MESSAGE_IGNORE_INSERTS
      
      FormatMessage dwFlags:=p_lngFlags, _
                    lpSource:=ByVal 0&, _
                    dwMessageId:=ErrorCode, _
                    dwLanguageId:=0&, _
                    lpBuffer:=p_strBuffer, _
                    nSize:=Len(p_strBuffer), _
                    Arguments:=ByVal 0&
   
   End If
   
   ' ------------------------------------------
   ' Strip the last null, then the last CrLf
   '     pair, if it exists
   ' ------------------------------------------
   p_lngPos = InStr(p_strBuffer, vbNullChar)
   If p_lngPos > 0 Then
      p_strBuffer = Left$(p_strBuffer, p_lngPos - 1)
   End If
   If Right$(p_strBuffer, 2) = Chr$(13) & Chr$(10) Then
      p_strBuffer = Mid$(p_strBuffer, 1, Len(p_strBuffer) - 2)
   End If
   
   ' ------------------------------------------
   ' Set the return value
   ' ------------------------------------------
   ReturnApiErrString = p_strBuffer

End Function

' *******************************************************
' Routine Name : (PRIVATE in MODULE) Function GetMsgTableResEntry
' Written By   : Brad Martinez (modified by L.J. Johnson)
' Programmer   : Brad Martinez (modified by L.J. Johnson)
' Date Written : 09/23/2001 -- 11:53:30
' Inputs       : xi_abytResource():Byte  - A byte array of the data from the resource file
'              : ByVal xi_lngErrNum:Long - The error number
' Outputs      : String --
' Description  : Source from Brad Martinez (modified by L.J. Johnson)
'              : Original at:  http://www.mvps.org/btmtz/_misc/netmsg.zip
'              : This will take a byte array (from the resource file)
'              :     and translate that passed error # into a text string
' Called By    : ReturnApiErrString()
' *******************************************************
Private Function GetMsgTableResEntry(xi_abytResource() As Byte, _
                                     ByVal xi_lngErrNum As Long) As String
On Error Resume Next                   ' Don't accept an error here
   Dim p_lngLenResource                As Long
   Dim p_lngNumBlocks                  As Long
   Dim p_lngLoop                       As Long
   Dim p_lngInnerLoop                  As Long
   Dim p_lngOffsetBlock                As Long
   Dim p_lngOffsetEntry                As Long
   Dim p_typMsgResBlock                As MESSAGE_RESOURCE_BLOCK
   Dim p_typMsgResEntry                As MESSAGE_RESOURCE_ENTRY
   Const CB_DWORD                      As Long = 4&
  
   ' ------------------------------------------
   ' Get the size of the resource byte array
   ' ------------------------------------------
   p_lngLenResource = (UBound(xi_abytResource) - LBound(xi_abytResource)) + 1
   
   ' ------------------------------------------
   ' Validate the size of the array, and get its
   '     count of message table blocks (blocks
   '     are groups of contiguously valued message
   '     table entries which are grouped at the
   '     beginning of the resource [after the
   '     block count])
   ' ------------------------------------------
   If (p_lngLenResource <= CB_DWORD) Then
      GetMsgTableResEntry = vbNullString
      
   ElseIf (p_lngLenResource > CB_DWORD) Then
      
      CopyMem pTo:=p_lngNumBlocks, _
              uFrom:=xi_abytResource(0), _
              lSize:=CB_DWORD
    
      For p_lngLoop = 0 To p_lngNumBlocks - 1
         
         ' ------------------------------------
         ' Calculate the offset to the current
         '     block, validate that the current
         '     block lies within the size of the
         '     resource, and fill the block's struct
         ' ------------------------------------
         p_lngOffsetBlock = CB_DWORD + (p_lngLoop * CB_MRB)
         
         If (p_lngLenResource <= (p_lngOffsetBlock + CB_MRB)) Then
            Exit Function
         End If
         
         CopyMem p_typMsgResBlock, _
                 xi_abytResource(p_lngOffsetBlock), _
                 CB_MRB
         
         ' ------------------------------------
         ' If the specified error # lies within
         '     this block...
         ' ------------------------------------
         If ((xi_lngErrNum >= p_typMsgResBlock.LowId) And _
            (xi_lngErrNum <= p_typMsgResBlock.HighId)) Then
           
            ' ---------------------------------
            ' Calculate the next entry's offset
            '     in the resource, and start
            '     walking the current block's
            '     enties...
            ' ---------------------------------
            p_lngOffsetEntry = p_typMsgResBlock.OffsetToEntries
            
            For p_lngInnerLoop = p_typMsgResBlock.LowId To p_typMsgResBlock.HighId
              
               ' ------------------------------
               ' Validate that the current entry
               '     lies with the size of the
               '     resource, and fill its struct
               ' ------------------------------
               If (p_lngLenResource <= (p_lngOffsetEntry + CB_MRE)) Then
                  Exit Function
               End If
               
               CopyMem pTo:=p_typMsgResEntry, _
                       uFrom:=xi_abytResource(p_lngOffsetEntry), _
                       lSize:=CB_MRE
               
               If (xi_lngErrNum = p_lngInnerLoop) Then
                  ' ---------------------------
                  ' If the specified entry matches
                  '     this entry's value, return
                  '     its description
                  ' ---------------------------
                  If p_typMsgResEntry.Flags Then
                     GetMsgTableResEntry = PointerToStringW(VarPtr(xi_abytResource(p_lngOffsetEntry)) + CB_MRE_TEXTOFFSET)
                  Else
                     GetMsgTableResEntry = PointerToStringA(VarPtr(xi_abytResource(p_lngOffsetEntry)) + CB_MRE_TEXTOFFSET)
                  End If
                  
                  Exit Function
               
               Else
                  ' ---------------------------
                  ' Otherwise add the size of
                  '     the current entry so it
                  '     points to the next entry
                  ' ---------------------------
                  p_lngOffsetEntry = p_lngOffsetEntry + p_typMsgResEntry.Length
               
               End If
              
            Next p_lngInnerLoop
        
        End If   ' ((xi_lngErrNum >= p_typMsgResBlock.LowId) Or (xi_lngErrNum <= p_typMsgResBlock.HighId))
        
      Next p_lngLoop
   
   End If   ' (p_lngLenResource > 4)
  
End Function
