Attribute VB_Name = "modMain"
' ************************************************************
' Copyright  1995-2001 Slightly Tilted Software
' All rights reserved
' You're absolutely free to use these resources within your
'     own applications, but you may not redistribute them
'     (as source) in any manner whatsoever, whether for profit
'     or not.
' The only legitimate source for the original source code is
'     at the VBPJ site and my own web site at:
'     http://www.SlightlyTiltedSoftware.com
' ************************************************************

' ---------------------------------------------
' Module    : modMain.BAS
' By        : L.J. Johnson       Date: 05-28-2001
' Comments  : General utility functions
' ---------------------------------------------
Option Explicit

Public Const MAX_TRIES                 As Long = 5

' ---------------------------------------------
' Status Codes
' ---------------------------------------------
Private Const INVALID_HANDLE_VALUE     As Long = -1&
Public Const ERROR_SUCCESS             As Long = 0&
Public Const MAX_PATH                  As Long = 260&
Public Const cnstErrBase               As Long = vbObjectError + 1000

' ---------------------------------------------
' General API functions
' ---------------------------------------------
Public Declare Function StrLenA _
   Lib "kernel32" Alias "lstrlenA" _
   (ByVal lpString As Any) As Long
Public Declare Function StrLenW _
   Lib "kernel32" Alias "lstrlenW" _
   (ByVal Ptr As Long) As Long
Public Declare Function lstrcpy _
   Lib "kernel32" Alias "lstrcpyA" _
   (ByVal lpString1 As Any, _
    ByVal lpString2 As Any) As Long
Public Declare Sub CopyMem _
   Lib "kernel32" Alias "RtlMoveMemory" _
   (pTo As Any, _
    uFrom As Any, _
    ByVal lSize As Long)
Public Declare Function GlobalAlloc _
   Lib "kernel32" _
   (ByVal wFlags As Long, _
    ByVal dwBytes As Long) As Long
Public Declare Function GlobalFree _
   Lib "kernel32" _
   (ByVal hMem As Long) As Long

Public Const GMEM_FIXED                As Long = &H0
Public Const GMEM_ZEROINIT             As Long = &H40
Public Const GPTR                      As Long = (GMEM_FIXED Or GMEM_ZEROINIT)

' ---------------------------------------------
'
' ---------------------------------------------
Private Declare Function GetShortPathName _
   Lib "kernel32" Alias "GetShortPathNameA" _
   (ByVal lpszLongPath As String, _
    ByVal lpszShortPath As String, _
    ByVal nBufferLength As Long) As Long
Private Declare Function MultiByteToWideChar _
   Lib "kernel32" _
   (ByVal codepage As Long, _
    ByVal dwFlags As Long, _
    lpMultiByteStr As Any, _
    ByVal cchMultiByte As Long, _
    lpWideCharStr As Any, _
    ByVal cchWideChar As Long) As Long

Private Const CP_ACP                   As Long = 0&   ' ANSI code page

' -------------------------------------------------
' The types of events that can be logged.
' -------------------------------------------------
Public Const EVENTLOG_SUCCESS          As Long = &H0
Public Const EVENTLOG_ERROR_TYPE       As Long = &H1
Public Const EVENTLOG_WARNING_TYPE     As Long = &H2
Public Const EVENTLOG_INFORMATION_TYPE As Long = &H4
Public Const EVENTLOG_AUDIT_SUCCESS    As Long = &H8
Public Const EVENTLOG_AUDIT_FAILURE    As Long = &H10

' -------------------------------------------------
' Constants for different types of events
' -------------------------------------------------
Public Const Event_Type_Info           As String = "Information"
Public Const Event_Type_Warning        As String = "Warning"
Public Const Event_Type_Error          As String = "Error"
Public Const Event_Type_Success_Audit  As String = "Success Audit"
Public Const Event_Type_Failure_Audit  As String = "Failure Audit"

' -------------------------------------------------
' Constants for the type of log you want to look
'   at.  Note that user program will need to use
'   these constants also
' -------------------------------------------------
Public Const EVNT_SYSTEM               As String = "System"
Public Const EVNT_APP                  As String = "Application"
Public Const EVNT_SECURITY             As String = "Security"

' -------------------------------------------------
' Custom error return codes for the various class
'   modules
' -------------------------------------------------
Public Const ERR_LOGTYPE_NOT_SET       As Long = 1011&
Public Const ERR_SOURCENAME_NOT_SET    As Long = 1012&
Public Const ERR_BAD_INDEX             As Long = 1013&
Public Const ERR_FAILED_OPEN_REGISTRY_KEY As Long = 1014&
Public Const ERR_FAILED_READ_REGISTRY_KEY As Long = 1015&
Public Const ERR_RESOURCE_DATA_NOT_FOUND  As Long = 1016&
Public Const ERR_READING_EVENT_LOG     As Long = 1017&
Public Const ERR_LOG_NOT_OPENED        As Long = 1018&
Public Const ERR_FAILED_SET_LOG_TYPE   As Long = 1019&
Public Const ERR_FAILED_FORMAT_MSG     As Long = 1020&
Public Const ERR_BAD_SERVER_NAME       As Long = 1021&
Public Const ERR_READ_EVENT_RECORD     As Long = 1022&

' --------------------------------------------
'
' --------------------------------------------
Private Type FILETIME
   dwLowDateTime                       As Long
   dwHighDateTime                      As Long
End Type

Private Type SYSTEMTIME
  wYear                                As Integer
  wMonth                               As Integer
  wDayOfWeek                           As Integer
  wDay                                 As Integer
  wHour                                As Integer
  wMinute                              As Integer
  wSecond                              As Integer
  wMilliseconds                        As Integer
End Type

Private Declare Function SystemTimeToFileTime _
   Lib "kernel32" _
   (lpSystemTime As SYSTEMTIME, _
    lpFileTime As FILETIME) As Long
Private Declare Function FileTimeToLocalFileTime _
   Lib "kernel32" _
   (lpFileTime As FILETIME, _
    lpLocalFileTime As FILETIME) As Long
Private Declare Function FileTimeToSystemTime _
   Lib "kernel32" _
   (lpFileTime As FILETIME, _
    lpSystemTime As SYSTEMTIME) As Long
Private Declare Function VariantTimeToSystemTime _
   Lib "oleaut32" _
   (ByVal vtime As Double, _
    psystime As SYSTEMTIME) As Long
Private Declare Function SystemTimeToVariantTime _
   Lib "oleaut32" _
   (psystime As SYSTEMTIME, _
    pvtime As Double) As Long

Private Declare Function WideCharToMultiByte _
   Lib "kernel32" _
   (ByVal codepage As Long, _
    ByVal dwFlags As Long, _
    lpWideCharStr As Any, _
    ByVal cchWideChar As Long, _
    lpMultiByteStr As Any, _
    ByVal cchMultiByte As Long, _
    ByVal lpDefaultChar As String, _
    ByVal lpUsedDefaultChar As Long) As Long

' *******************************************************
' Routine Name : (PUBLIC in MODULE) Function DateToLocalDate
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Written : 05/29/2001 -- 15:37:35
' Inputs       : xi_dteTimes:Date -
' Outputs      : N/A
' Description  : Converts a date/time from UCT (Universal
'              :     Coordiated Time) to the local time
'              :     of this machine
'              : Dates for the event log items are stored
'              :     in UCT format
'              : Thanks to Brad Martinez for this code
' Called By    : *****
' *******************************************************
Public Function DateToLocalDate(ByVal xi_dteTimes As Date) As Date
   Dim p_typSystemTime                 As SYSTEMTIME
   Dim p_typFileTime                   As FILETIME
   Dim p_typLocalFileTime              As FILETIME
   Dim p_dblReturn                     As Double
  
   VariantTimeToSystemTime CDbl(xi_dteTimes), p_typSystemTime
   SystemTimeToFileTime p_typSystemTime, p_typFileTime
   FileTimeToLocalFileTime p_typFileTime, p_typLocalFileTime
   FileTimeToSystemTime p_typLocalFileTime, p_typSystemTime
   SystemTimeToVariantTime p_typSystemTime, p_dblReturn
   DateToLocalDate = p_dblReturn
  
End Function

' *******************************************************
' Routine Name : (PUBLIC in MODULE) Function PointerToStringA
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Written : 05/29/2001 -- 15:52:43
' Inputs       : ByVal xi_lngPtr:Long                 - Pointer to an ASCII string
'              : Optional ByVal xi_lngStrLen:Long = 0 - When you want just *part* of the string
' Outputs      : N/A
' Description  : Return a string given a pointer to a
'              :     string
'              : Thanks to Brad Martinez for this code --
'              :     had to add an optional parameter to
'              :     handle items with included nulls
' Called By    : *****
' *******************************************************
Public Function PointerToStringA(ByVal xi_lngPtr As Long, _
                               Optional ByVal xi_lngStrLen As Long = 0) As String
On Error Resume Next                   ' Don't accept an error here
   Dim p_lngLen                        As Long
   Dim p_lngRtn                        As Long
   Dim p_lngNullPos                    As Long
   Dim p_strStringValue                As String
   
   p_lngLen = StrLenA(xi_lngPtr)
   
   If xi_lngPtr > 0 And p_lngLen > 0 Then
   
      p_strStringValue = Space$(p_lngLen + 1)
      p_lngRtn = lstrcpy(p_strStringValue, xi_lngPtr)
      p_lngNullPos = InStr(1, p_strStringValue, vbNullChar)
      
      If p_lngNullPos > 0 Then
         ' Lose the null terminator...
         PointerToStringA = Left$(p_strStringValue, p_lngNullPos - 1)
      Else
         ' Just pass the string...
         PointerToStringA = p_strStringValue
      End If
      
   Else
      PointerToStringA = ""
   End If

End Function

' *******************************************************
' Routine Name : (PUBLIC in MODULE) Function PointerToStringW
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Written : 09/23/2001 -- 11:49:49
' Inputs       : ByVal xi_lngPtr:Long - Pointer to a Unicode string
' Outputs      : String --
' Description  : Takes a pointer to an UNICODE string and
'              :     converts it into the string
' Called By    : *****
' *******************************************************
Public Function PointerToStringW(ByVal xi_lngPtr As Long) As String
On Error Resume Next                   ' Don't accept an error here
   Dim p_abytBuffer()                  As Byte
   Dim p_lngLen                        As Long
   Dim p_strTmp                        As String
   Dim p_lngRtn                        As Long
   
   ' ------------------------------------------
   ' Get the length of the string from the pointer
   ' ------------------------------------------
   If xi_lngPtr <> 0 Then
      p_lngLen = StrLenW(xi_lngPtr) * 2
   Else
      p_lngLen = 0
   End If
   
   ' ------------------------------------------
   ' Now convert to a string
   ' ------------------------------------------
   If xi_lngPtr <> 0 Then
      
      If p_lngLen > 0 Then
         ReDim p_abytBuffer(0 To (p_lngLen - 1)) As Byte
         CopyMem p_abytBuffer(0), ByVal xi_lngPtr, p_lngLen
         p_strTmp = p_abytBuffer
      Else
         p_strTmp = ""
      End If
   
   End If
   
   ' ------------------------------------------
   ' Set the return value
   ' ------------------------------------------
   PointerToStringW = p_strTmp
   
End Function

' *******************************************************
' Routine Name : (PUBLIC in MODULE) Function PointerToLong
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Written : 09/23/2001 -- 12:15:22
' Inputs       : ByVal xi_lngPtrLong:Long - Pointer to a long
' Outputs      : Long --
' Description  : Converts a pointer to a long into a long
' Called By    : *****
' *******************************************************
Public Function PointerToLong(ByVal xi_lngPtrLong As Long) As Long
On Error Resume Next                   ' Don't accept an error here
      
   If xi_lngPtrLong <> 0 Then
      CopyMem pTo:=PointerToLong, _
              uFrom:=ByVal xi_lngPtrLong, _
              lSize:=Len(xi_lngPtrLong)
   Else
      PointerToLong = 0
   End If
   
End Function

' *******************************************************
' Routine Name : (PUBLIC in MODULE) Function ClearLastCrLf
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Written : 05/29/2001 -- 15:48:54
' Inputs       : xi_strTmpMsg:String -
' Outputs      : STRING:
' Description  : Clear any trailing CRs, LFs or CRLFs
'              :     from the passed string
' Called By    : *****
' *******************************************************
Public Function ClearLastCrLf(ByVal xi_strTmpMsg As String) As String
   Dim p_strTmpMsg                     As String
   
   ' ----------------------------------------------
   ' Make a temporary copy of the passed string
   ' ----------------------------------------------
   p_strTmpMsg = Trim$(xi_strTmpMsg)
   
   ' ----------------------------------------------
   ' Trim any trailing nulls
   ' ----------------------------------------------
   If Right$(p_strTmpMsg, 1) = vbNullChar Then
      p_strTmpMsg = Mid$(p_strTmpMsg, 1, Len(p_strTmpMsg) - 1)
   End If
   
   ' ----------------------------------------------
   ' Trim any trailing Cr's, Lf's, or CrLf's
   ' ----------------------------------------------
   If Right$(p_strTmpMsg, 2) = vbCrLf Then
      p_strTmpMsg = Mid$(p_strTmpMsg, 1, Len(p_strTmpMsg) - 2)
   ElseIf Right$(p_strTmpMsg, 1) = vbLf Then
      p_strTmpMsg = Mid$(p_strTmpMsg, 1, Len(p_strTmpMsg) - 1)
   ElseIf Right$(p_strTmpMsg, 1) = vbCr Then
      p_strTmpMsg = Mid$(p_strTmpMsg, 1, Len(p_strTmpMsg) - 1)
   End If

   ' ----------------------------------------------
   ' Set the return value
   ' ----------------------------------------------
   ClearLastCrLf = p_strTmpMsg
   
End Function

' *******************************************************
' Routine Name : (PUBLIC in MODULE) Function GetShortName
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Written : 09/23/2001 -- 12:12:32
' Inputs       : ByVal xi_strLongPathName:String - Long path name
' Outputs      : String --
' Description  : Converts a long path name into a short
'              :     (8.3) path name
' Called By    : *****
' *******************************************************
Public Function GetShortName(ByVal xi_strLongPathName As String) As String
   Dim p_strBuffer                     As String
   Dim p_lngRtn                        As Long
   
   p_strBuffer = Space$(MAX_PATH)
   If Right$(xi_strLongPathName, 1) = "\" Then
      xi_strLongPathName = Mid$(xi_strLongPathName, 1, Len(xi_strLongPathName) - 1)
   End If
   
   p_lngRtn = GetShortPathName(lpszLongPath:=xi_strLongPathName, _
                               lpszShortPath:=p_strBuffer, _
                               nBufferLength:=Len(p_strBuffer))
   If p_lngRtn > 0 Then
      GetShortName = Trim$(Mid$(p_strBuffer, 1, p_lngRtn))
   Else
      GetShortName = ""
   End If
End Function

' *******************************************************
' Routine Name : (PUBLIC in MODULE) Function TrimNull
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Written : 09/23/2001 -- 12:20:42
' Inputs       : ByVal xi_strPassed:String -
' Outputs      : String --
' Description  : Truncate input string at first null.
'              : If no nulls, perform ordinary Trim.
' Called By    : *****
' *******************************************************
Public Function TrimNull(ByVal xi_strPassed As String) As String
   Dim p_lngPos                        As Long
   
   p_lngPos = InStr(xi_strPassed, vbNullChar)
   
   Select Case p_lngPos
      Case Is > 1
         TrimNull = Left$(xi_strPassed, p_lngPos - 1)
      Case 1
         TrimNull = vbNullString
      Case 0
         TrimNull = Trim$(xi_strPassed)
   End Select
   
End Function
