VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cOsVersion"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' ************************************************************
' Copyright  1995-2001 Slightly Tilted Software
' All rights reserved
' You're absolutely free to use these resources within your
'     own applications, but you may not redistribute them
'     (as source) in any manner whatsoever, whether for profit
'     or not.
' The only legitimate source for the original source code is
'     at the VBPJ site and my own web site at:
'     http://www.SlightlyTiltedSoftware.com
' ************************************************************

' ---------------------------------------------
' Module    : cOsVersion.cls
' By        : L.J. Johnson       Date: 09-23-2001
' Comments  : Determine the Operating System version
' ---------------------------------------------
Option Explicit

' --------------------------------------------
' API calls
' --------------------------------------------
Private Declare Function GetVersionEx _
   Lib "kernel32" Alias "GetVersionExA" _
   (lpVersionInformation As OSVERSIONINFO) As Long
Private Type OSVERSIONINFO
   dwOSVersionInfoSize                 As Long
   dwMajorVersion                      As Long
   dwMinorVersion                      As Long
   dwBuildNumber                       As Long
   dwPlatformId                        As Long
   szCSDVersion                        As String * 128
End Type

Private Enum enmPlatform
   VER_PLATFORM_UNKNOWN = -1&
   VER_PLATFORM_WIN32s = 0&
   VER_PLATFORM_WIN32_NT = 2&
   VER_PLATFORM_WIN32_WINDOWS = 1&
End Enum

Private m_typOSVersionInfo             As OSVERSIONINFO

Public Enum enmValidOSs
   Invalid_OS = 0&
   Valid_NT_Pre40 = 1&
   Valid_NT40 = 2&
   Valid_W2K = 3&
   Valid_XP = 4&
   Valid_Win95 = 5&
   Valid_Win98 = 6&
   Valid_WinME = 7&
End Enum

Private m_blnIsValidOS                 As Boolean
Private m_strVersionText               As String
Private m_strErrMsg                    As String
Private m_typOS                        As enmValidOSs

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Function IsValidOS
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Written : 09/23/2001 -- 13:47:56
' Inputs       : N/A
' Outputs      : Boolean --
' Description  : Returns TRUE if a valid OS, FALSE otherwise
' Called By    : *****
' *******************************************************
Public Function IsValidOS() As Boolean
   IsValidOS = m_blnIsValidOS
End Function

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Function TypeValidOS
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Written : 09/23/2001 -- 13:48:25
' Inputs       : N/A
' Outputs      : enmValidOSs --
' Description  : Returns the type of OS system (public enum value)
' Called By    : *****
' *******************************************************
Public Function TypeValidOS() As enmValidOSs
   TypeValidOS = m_typOS
End Function

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Function OS_TypeError
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Written : 09/23/2001 -- 13:54:26
' Inputs       : N/A
' Outputs      : String --
' Description  : If there's an error, you can get the
'              :     text for that error here.
' Called By    : *****
' *******************************************************
Public Function OS_TypeError() As String
   OS_TypeError = m_strErrMsg
End Function

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Function OS_VersionText
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Written : 09/23/2001 -- 13:48:58
' Inputs       : N/A
' Outputs      : String --
' Description  : Returns a text string for the current OS
' Called By    : *****
' *******************************************************
Public Function OS_VersionText() As String
   OS_VersionText = m_strVersionText
End Function

' *******************************************************
' Routine Name : (PRIVATE in CLASS) Sub GetOSVersion
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Written : 09/23/2001 -- 13:49:40
' Inputs       : N/A
' Outputs      : N/A
' Description  : Determine the type and version # of the
'              :     current OS
' Called By    : Class_Initialize
' *******************************************************
Private Sub GetOSVersion()
   Dim p_lngPos                        As Long
   Dim p_strVersion                    As String
   Dim p_lngMajorVer                   As Long
   Dim p_lngMinorVer                   As Long
   Dim p_intBuildNum                   As Integer
   Dim p_intServicePack                As Integer
   
   ' ------------------------------------------
   ' Set the length and make API call
   ' ------------------------------------------
   m_typOSVersionInfo.dwOSVersionInfoSize = Len(m_typOSVersionInfo)
   GetVersionEx m_typOSVersionInfo
   
   ' ------------------------------------------
   ' Trim the null from the Version string
   ' ------------------------------------------
   p_lngPos = InStr(1, m_typOSVersionInfo.szCSDVersion, vbNullChar)
   If p_lngPos > 0 Then
      p_strVersion = Trim$(Mid$(m_typOSVersionInfo.szCSDVersion, 1, p_lngPos - 1))
   Else
      p_strVersion = ""
   End If
   
   ' ------------------------------------------
   ' Get the details
   ' ------------------------------------------
   p_lngMajorVer = m_typOSVersionInfo.dwMajorVersion
   p_lngMinorVer = m_typOSVersionInfo.dwMinorVersion
   p_intBuildNum = WordLow(m_typOSVersionInfo.dwBuildNumber)
   If Len(p_strVersion) > 0 Then
      p_intServicePack = Val(Right$(p_strVersion, 1))
   Else
      p_intServicePack = 0
   End If
   
   ' ------------------------------------------
   ' Create a combined version string
   ' ------------------------------------------
   m_strVersionText = CStr(p_lngMajorVer) & "." & _
                      CStr(p_lngMinorVer)
   
   If p_intBuildNum > 0 Then
      m_strVersionText = m_strVersionText & " Build " & CStr(p_intBuildNum)
   End If
   
   If p_intServicePack > 0 Then
      m_strVersionText = m_strVersionText & " SP " & CStr(p_intServicePack)
   End If
   
   ' ------------------------------------------
   ' Branch initially based on the Platform ID
   ' ------------------------------------------
   Select Case m_typOSVersionInfo.dwPlatformId
      
      ' ---------------------------------------
      ' It's Win9x
      ' ---------------------------------------
      Case VER_PLATFORM_WIN32_WINDOWS
         If p_lngMinorVer < 10 Then
            m_strVersionText = "Windows 95 " & m_strVersionText
            m_strErrMsg = ""
            m_blnIsValidOS = True
            m_typOS = Valid_Win95
         ElseIf (p_lngMinorVer >= 10) And (p_lngMinorVer < 90) Then
            m_strVersionText = "Windows 98 " & m_strVersionText
            m_strErrMsg = ""
            m_blnIsValidOS = True
            m_typOS = Valid_Win98
         ElseIf p_lngMinorVer >= 90 Then
            m_strVersionText = "Windows ME " & m_strVersionText
            m_strErrMsg = ""
            m_blnIsValidOS = True
            m_typOS = Valid_WinME
         End If
         
      ' ---------------------------------------
      ' It's NT+
      ' ---------------------------------------
      Case VER_PLATFORM_WIN32_NT
         
         ' ------------------------------------
         ' Pre-NT 4.0
         ' ------------------------------------
         If p_lngMajorVer = 3 And p_lngMinorVer < 5 Then
            m_strErrMsg = "You are running " & m_strVersionText & _
                          ". You must be running at least NT 3.5"
            m_blnIsValidOS = False
            m_typOS = Invalid_OS
            
         ' ------------------------------------
         ' NT 3.50+
         ' ------------------------------------
         ElseIf (p_lngMajorVer = 3) And (p_lngMinorVer >= 5) Then
            m_strVersionText = "NT, Pre Version 4.0" & m_strVersionText
            m_strErrMsg = ""
            m_blnIsValidOS = True
            m_typOS = Valid_NT_Pre40
            
         ' ------------------------------------
         ' NT 4.0
         ' ------------------------------------
         ElseIf p_lngMajorVer = 4 Then
            m_strVersionText = "NT 4.0" & m_strVersionText
            m_strErrMsg = ""
            m_blnIsValidOS = True
            m_typOS = Valid_NT40
            
         ' ------------------------------------
         ' Win2000
         ' ------------------------------------
         ElseIf (p_lngMajorVer = 5) And (p_lngMinorVer = 0) Then
            m_strVersionText = "Windows 2000" & m_strVersionText
            m_strErrMsg = ""
            m_blnIsValidOS = True
            m_typOS = Valid_W2K
            
         ' ------------------------------------
         ' XP
         ' ------------------------------------
         ElseIf (p_lngMajorVer = 5) And (p_lngMinorVer > 0) Then
            m_strVersionText = "Windows XP" & m_strVersionText
            m_strErrMsg = ""
            m_blnIsValidOS = True
            m_typOS = Valid_XP
            
         Else
            m_strErrMsg = "Unknown operating system: " & m_strVersionText
            m_blnIsValidOS = False
            m_typOS = Invalid_OS
         End If
         
      ' ---------------------------------------
      ' MS must have added a new OS....
      ' ---------------------------------------
      Case Else
         m_strErrMsg = "Unknown operating system: " & m_strVersionText
         m_blnIsValidOS = False
         m_typOS = Invalid_OS

   End Select
   
End Sub

' *******************************************************
' Routine Name : (PRIVATE in CLASS) Function WordLow
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Written : 09/23/2001 -- 13:51:05
' Inputs       : ByVal xi_lngInput:Long -
' Outputs      : Integer --
' Description  : Code is replicated here so the class
'              :     will be loosely coupled (not dependent
'              :     on other classes)
' Called By    : *****
' *******************************************************
Private Function WordLow(ByVal xi_lngInput As Long) As Integer
   
   If (xi_lngInput And &HFFFF&) > &H7FFF Then
      WordLow = (xi_lngInput And &HFFFF&) - &H10000
   Else
      WordLow = xi_lngInput And &HFFFF&
   End If
   
End Function

' *******************************************************
' Routine Name : (PRIVATE in CLASS) Sub Class_Initialize
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Written : 09/23/2001 -- 13:50:18
' Inputs       : N/A
' Outputs      : N/A
' Description  : Determine the type of OS *once* (when
'              :     the class is insubstantiate)
' Called By    : *****
' *******************************************************
Private Sub Class_Initialize()
   GetOSVersion
End Sub
